#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import subprocess
import network_util
import sys
sys.path.append("/linuxcash/cash/tools/tools_avail")
from py_tools.dialog import dialog
from py_tools.netconfig.simple_network_wizard import NetworkWizard
from py_tools.logger import logger

log = logger.Logger()

def manualMenu(d, eth):
	"""
	Меню ручной настройки
	"""
	return d.menu(
		"Интерфейс: " + eth['name'] + " (" + eth['type'] + ")",
		width=60, height=16, menu_height=9,
		choices=[
			("choose", "Выбрать сетевую карту"),
			("edit", "Настройка сетевых параметров"),
			("connect", "Подключить соединение"),
			("disconnect", "Отключить соединение"),
			("ping", "Проверить соединение"),
			("source", "Посмотреть содержимое interfaces"),
			("save", "Сохранить"),
			("quit", "Выйти")
		])

def mainMenu(d):
	"""
	Главное меню
	"""
	return d.menu(
		"Управление соединениями",
		width=60, height=11, menu_height=4,
		choices=[
			("wizard", "Мастер настройки сети"),
			("manual", "Ручная настройка сети"),
			("info", "Сетевые адреса"),
			("quit", "Выйти")
		])
	

def createEditMenu(d, eth):
	"""
	Меню редактирования сетевых параметров
	"""
	items = []
	if eth['type'] == "Wireless":
		items = [("search", "Поиск доступных точек доступа"),
				("ssid", "Имя точки доступа"),
				("scan", "Видимость точки доступа"),
				("security", "Тип защиты"),
				("password", "Пароль")]
	items.append(("mode", "Способ подключения"))
	if eth['config']['mode'] == "static":
		items.append(("address", "Редактировать IP"))
	items.append(("quit", "Выйти"))
	
	return d.menu(
				"Настройка интерфейса: " + eth['name'] + " (" + eth['type'] + ")",
				width=60, height=len(items) + 7, menu_height=len(items),
				choices = items)


def chooseDeviceDialog(d, devices):
	"""
	Диалог выбора сетевого интерфейса
	"""
	return d.radiolist(
		"Какую сетевую карту требуется настроить?",
		width=60, height=len(devices) + 7,
		choices=devices)


def typeOfSetupDialog(d, eth):
	"""
	Диалог выбора типа настройки сетевого интерфейса
	"""
	values = [("auto", "Автоматическая настройка сети", 0),
		("manual", "Настройка сети вручную", 0)]
	if eth['config']['mode'] == "dhcp":
		values[0] = values[0][:-1] + (1,)
	elif eth['config']['mode'] == 'static':
		values[1] = values[1][:-1] + (1,)

	return d.radiolist(
		text = "Выберите способ настройки интерфейса:",
		height=10, width=54,
		choices = values)


def manualSetup(d, selectedEth):
	"""
	Настройка интерфейса selectedEth статически
	"""
	currentConfig = {}
	for i in selectedEth['config']:
		currentConfig[i] = selectedEth['config'][i]
	isReady = False
	while not isReady:
		(code, tag) = manualSetupDialog(d, currentConfig)
		if code == d.DIALOG_OK:
			#проверяем введенные настройки
			currentConfig['address'] = tag[0]
			currentConfig['netmask'] = tag[1]
			currentConfig['gateway'] = tag[2]
			isReady = True
			if len(currentConfig['address']) > 0:
				if len(currentConfig['netmask']) == 0:
					currentConfig['netmask'] = "255.255.255.0"
			else:
				log.warn("Ошибка валидности адреса интерфейса")
				d.msgbox("Вы должны ввести адрес интерфейса", height=6)
				isReady = False

			if isReady:
				for i in currentConfig:
					selectedEth['config'][i] = currentConfig[i]
			log.info("Интерфейс %s" % selectedEth['name'] + " успешно настроен \n%s" % selectedEth)
		else:
			isReady = True


def manualSetupDialog(d, config):
	"""
	Диалог настройки интерфейса статически
	"""
	return d.form(
		text = "Введите параметры сетевого подключения",
		height=10, width=54, form_height=3,
		fields = [("IP-адрес:", config['address'], 16, 15),
			("Маска сети:", config['netmask'], 16, 15),
			("Шлюз:", config['gateway'], 16, 15)],
		title = "Настройка сети вручную")


def chooseEth(d, eths, selectedEth):
	"""
	Выбор сетевого интерфейса из списка интерфесов eths
	"""
	if len(eths) >= 1:
		devsForDialog = []
		for dev in eths:
			if dev == selectedEth:
				devsForDialog.append((dev['name'], dev['description'], 1))
			else:
				devsForDialog.append((dev['name'], dev['description'], 0))
		(code, tag) = chooseDeviceDialog(d, devsForDialog)
		if code == d.DIALOG_OK:
			for eth in eths:
				if eth['name'] == tag:
					return eth
		else:
			return selectedEth
	return None


def pingDialog(d, eth, ping_settings):
	"""
	Диалог проверки соединения
	"""
	if eth['config']['mode'] == "static" and ping_settings['ip'] is None:
		ping_settings['ip'] = eth['config']['address']
	if ping_settings['ip'] is None:
		ping_settings['ip'] = ""
	(code, tag) = d.form(
		text = "Введите параметры для проверки командой ping",
		height=10, width=54, form_height=2,
		fields = [(u"IP-адрес:", ping_settings['ip'], 16, 60),
			(u"Кол-во пакетов:", ping_settings['count'], 16, 15)],
		title = "Проверка соединения")
	if code == d.DIALOG_OK:
		try:
			log.info("Старт проверки соединения")
			d.infobox("Подождите. Идет проверка соединения", height=7)
			ping_settings['ip'] = tag[0]
			ping_settings['count'] = tag[1]
			(code, (stdout, stderr)) = network_util.ping(ping_settings['ip'], ping_settings['count'])
			if code == 0:
				d.scrollbox(stdout, width=70, height=14)
			else:
				if len(stdout) > 0:
					d.scrollbox(stdout)
				elif len(stderr) > 0:
					d.scrollbox(stderr)
				else:
					log.warn("Ошибка проверки соединения")
					d.msgbox("Ошибка проверки соединения", height=6)
			log.info("Окончание проверки соединеиня")
		except KeyboardInterrupt:
			log.warn("Проверка соединения была прервана")
			d.msgbox("Проверка соединения была прервана", height=7)


def connectDialog(d, eth, filename):
	"""
	Диалог подключения интерфейса eth
	"""
	try:
		d.infobox("Подождите. Идет подключение соединения", height=7)
		network_util.disconnect(eth, filename)
		(code, (stdout, stderr)) = network_util.connect(eth, filename)
		if network_util.isConnected(eth['config']['mode'], code, stdout, stderr):
			log.info("Интерфейс %s" % eth['name'] + ". Соединение успешно установлено")
			d.msgbox("Соединение успешно установлено", height=7)
		else:
			log.warn("Интерфейс %s" % eth['name'] + ". Соединение не установлено")
			d.msgbox("Соединение не установлено", height=7)
	except KeyboardInterrupt:
		log.warn("Подключение было прервано")
		d.msgbox("Подключение было прервано", height=7)


def disconnectDialog(d, eth, filename):
	"""
	Диалог отключения интерфейса
	"""
	try:
		d.infobox("Подождите. Идет отключение соединения", height=8)
		(code, (stdout, stderr)) = network_util.disconnect(eth, filename)
		d.msgbox("Соединение разорвано", height=7)
	except KeyboardInterrupt:
		log.warn("Отключение было прервано")
		d.msgbox("Отключение было прервано", height=7)

def check_interface(eth):
	"""
	Проверить интерфейс на правильность заполнения
	"""
	if eth['config']['mode'] == 'dhcp':
		return True
	elif eth['config']['mode'] == 'static':
		if not eth['config']['address']:
			return False
		if not eth['config']['netmask']:
			return False
	else:
		return False
	return True

def save_settings(selectedEth, tmp_file, interfaces_file):
	"""
	Сохранить сетевые настройки в файл
	"""
	network_util.writePropertiesToInterfacesFile(selectedEth, tmp_file, interfaces_file)
	return True

def chooseScanType(d, eth):
	""" Выбор типа вещания имени точки доступа для Wireless интерфейса"""
	scan = eth['config']['scan']
	if not scan:
		scan = 'visible'
	(code, tag) = d.radiolist(
		text = "Точка доступа скрытая или видимая?",
		width=60, height=9,
		choices=[('visible', 'Видимая точка доступа', int(scan == 'visible')),
			('hidden', 'Скрытая точка доступа', int(scan == 'hidden'))])
	if code == d.DIALOG_OK:
		log.info("Тип вещания точки доступа %s" % eth['config']['ssid'] + " изменён c %s" % eth['config']['scan'] + " на %s" % tag)
		eth['config']['scan'] = tag
	
def chooseAP(d, eth):
	"""Выбор точки доступа из всех найденных при сканировании"""
	log.info("Поиск точек доступа...")
	d.infobox("Подождите. Программа выполняет поиск точек доступа.", height=6)
	aps = network_util.scanAccessPoints(eth)
	if not len(aps):
		log.info("Не обнаружено ни одной точки доступа.")
		d.msgbox("Не обнаружено ни одной точки доступа.", height=6)
		return
	aplist = []
	log.info("Найдены точки доступа %s" % aps)
	for ap in aps:
		aplist.append((ap,'Сигнал ' + str(aps[ap]) + '%', 0))
	(code, tag) = d.radiolist(
		text = 'Выбор точки доступа',
		width = 60, height = 11,
		choices = aplist)
	if code == d.DIALOG_OK:
		log.info("Выбрана точка доступа %s" % tag)
		eth['config']['ssid'] = tag
	
def setESSID(d, eth):
	"""Установка имени точки доступа"""
	ssid = eth['config']['ssid']
	if not ssid:
		ssid = ''
	(code, tag) = d.inputbox(
		"Отредактируйте или оставьте без изменения",
		title = 'Шаг 1.3: Установка имени точки доступа',
		width = 60, height = 9,
		init = ssid)
	if code == d.DIALOG_OK:
		log.info("Изменено имя точки доступа '%s" % eth['config']['ssid'] + "' на '%s" % tag + "'")
		eth['config']['ssid'] = tag
	
def chooseSecurity(d, eth):
	"""Установка типа защиты"""
	secur = eth['config']['security']
	if not secur:
		secur = 'wpa'	
	(code, tag) = d.radiolist(
		text = 'Установка типа защиты',
		width = 60, height = 11,
		choices = [('wpa', 'WPA шифрование', int(secur == 'wpa')),
				('wpa2', 'WPA2 шифрование', int(secur == 'wpa2')),
				('wpa/wpa2', 'Смешанный режим', int(secur == 'wpa/wpa2'))])
	if code == d.DIALOG_OK:
		log.info("Изменён тип защиты точки доступа %s" % eth['config']['ssid'] + " c %s" % eth['config']['security'] + " на %s" % tag)
		eth['config']['security'] = tag

def setPassword(d, eth):
	"""Установка пароля"""
	passw = eth['config']['pass']
	if not passw:
		passw = ''
	(code, tag) = d.inputbox(
		text = 'Установка пароля',
		width = 60, height = 11,
		init = passw)
	if code == d.DIALOG_OK:
		log.info("Пароль для точки доступа %s" % eth['config']['ssid'] + " изменён с %s" % eth['config']['pass'] + " на %s" % tag)
		eth['config']['pass'] = tag
	
def editSubMenu(d, eth):
	isExit = False
	while not isExit:
		(sm_code, sm_tag) = createEditMenu(d, eth)
		if sm_code == d.DIALOG_OK:
			if sm_tag == "mode":
				log.info("Выбор способа настройки интерфейса")
				(code, tag) = typeOfSetupDialog(d, eth)
				if code == d.DIALOG_OK:
					if tag == "manual":
						log.info("Выбран способ 'вручную'")
						eth['config']['mode'] = "static"
					elif tag == "auto":
						log.info("Выбран способ 'автоматически'")
						eth['config']['mode'] = "dhcp"
				log.info("Завершение выбора способа подключения")
			elif sm_tag == "search":
				log.info("Выбор точки доступа для %s" % eth['name'])
				chooseAP(d, eth)
				log.info("Конец выбора точки доступа")
			elif sm_tag == "ssid":
				log.info("Установка имени точки доступа %s" % eth['config']['ssid'])
				setESSID(d, eth)
				log.info("Завершение установки имени")
			elif sm_tag == "scan":
				log.info("Выбор типа вещания для точки доступа %s" % eth['config']['ssid'])
				chooseScanType(d, eth)
				log.info("Завершение выбора типа вещания")
			elif sm_tag == "security":
				log.info("Установка типа защиты для точки доступа %s" % eth['config']['ssid'])
				chooseSecurity(d, eth)
				log.info("Завершение установки типа защиты")
			elif sm_tag == "address":
				log.info("Настройка параметров интерфейса %s" % eth['name'] + " статически")
				manualSetup(d, eth)
				log.info("Завершение настройки статических параметров")
			elif sm_tag == "password":
				log.info("Установка пароля для точки доступа %s" % eth['config']['ssid'])
				setPassword(d, eth)
				log.info("Завершение установки пароля")
			elif sm_tag == "quit":
				(result, message) = network_util.checkProperties(eth)
				if not result:
					log.warn("Не указаны параметры:\n%s" % message)
					d.msgbox("Не указаны параметры:" + message)
				else:
					log.info("Параметры интерфейса %s" % eth['name'] + " успешно применены \n%s" % eth)
					return True			
		else:
			log.info("Отмена установки параметров")
			return False

def manualSubMenu(d):
	ping_settings = {
		'ip': None,
	    'count': "4"
	}
	network_util.loadDialog(d)
	eths = network_util.getNetworkDevices()
	selectedEth = None
	isSaved = False
	isChanged = False
	interfaces_file = "/etc/network/interfaces"
	tmp_file = "/tmp/interfaces_tmp_extended"

	if len(eths) == 0:
		network_util.noInterfaceDialog(d)
	else:
		selectedEth = eths[0]
		network_util.getPropertiesFromFile(selectedEth, interfaces_file)

	isExit = False
	try:
		while not isExit:
			(code, tag) = manualMenu(d, selectedEth)
			if code == d.DIALOG_OK:
				if tag == "choose":
					log.info("Выбор сетевой карты")
					newEth = chooseEth(d, eths, selectedEth)
					if newEth is not selectedEth:
						selectedEth = newEth
						ping_settings['ip'] = None
					log.info("Сетевая карта выбрана %s" % selectedEth['name'])
					network_util.getPropertiesFromFile(selectedEth, interfaces_file)
				elif tag == "edit":
					log.info("Настройка сетевых параметров")
					isChanged = editSubMenu(d, selectedEth)
					if isChanged:
						network_util.writePropertiesToInterfacesFile(selectedEth, interfaces_file, tmp_file)
					log.info("Завершение настройки сетевых параметров")
				elif tag == "source":
					log.info("Просмотр содержимого %s" % interfaces_file)
					d.textbox(interfaces_file)
					log.info("Выход из пункта 'Просмотр содержимого'")
				elif tag == "save":
					log.info("Сохранение настроек в конфигурационный файл")
					if isChanged == True:
						isSaved = save_settings(selectedEth, tmp_file, interfaces_file)
						d.msgbox("Настройки сохранены")
					else:
						log.info("Настройки не были изменены")
						d.msgbox("Настройки не были изменены")
					log.info("Выход из пункта 'Cохранение настроек в конфигурационный файл'")
				elif tag == "connect":
					log.info("Подключение соединения")
					connectDialog(d, selectedEth, tmp_file)
					log.info("Выход из пункта 'Подключение соединения'")
				elif tag == "disconnect":
					log.info("Отключение соединения")
					disconnectDialog(d, selectedEth, tmp_file)
					log.info("Выход из пункта 'Отключение соединения'")
				elif tag == "ping":
					log.info("Проверка соединения")
					pingDialog(d, selectedEth, ping_settings)
					log.info("Выход из пункта 'Проверка соединения'")
				else:
					if isSaved == False and isChanged == True:
						code = d.yesno("Вы не сохранили настройки. Хотите их сохранить?")
						if code == d.DIALOG_OK:
							isSaved = save_settings(selectedEth, tmp_file, interfaces_file)
							log.info("Сохранение настроек для сетевой карты:\n%s" % selectedEth + "\nв файл %s" % interfaces_file)
					isExit = True
			else:
				isExit = True
	finally:
		if os.path.exists(tmp_file):
			os.remove(tmp_file)
			log.info("Временный файл %s" % tmp_file + " удалён")

def main():
	"""
	main function
	"""
	log.info("Конфигурирование сети")
	d = dialog.Dialog()
	d.add_persistent_args(("--backtitle", "Настройка сети"))
	isExit = False
	while not isExit:
		(code, tag) = mainMenu(d)
		if code == d.DIALOG_OK:
			if tag == "manual":
				log.info("Ручная настройка сети")
				manualSubMenu(d)
				log.info("Завершение ручной настройки сети")
			elif tag == "wizard":
				log.info("Мастер настройки сети")
				wizard = NetworkWizard(d)
				wizard.run()
				log.info("Завершение мастера настройки сети")
			elif tag == "info":
				log.info("Просмотр сетевых адресов")
				d.msgbox(network_util.getIPs())
				log.info("Завершение просмотра сетевых адресов")
			else:
				isExit = True
		else:
			isExit = True
	log.info("Завершение конфигурирования сети")
	subprocess.call("clear")

if __name__ == "__main__":
	os.chdir(os.path.dirname(os.path.abspath(__file__)))
	try:
		main()
	except KeyboardInterrupt:
		log.warn("Неожиданное завершение конфигурирования сети")
